function [C] = nml_constant(n, m)

    global nml_constant_cache
    
    [ c_rows, c_cols ] = size(nml_constant_cache);
    if n <= c_rows && m <= c_cols
        C = nml_constant_cache(n, m);
        if C > 0
            return;
        end
    end
    c_rows_new = max(c_rows, n);
    c_cols_new = max(c_cols, m);
    if c_rows ~= c_rows_new || c_cols ~= c_cols_new
        tmp = zeros(c_rows_new, c_cols_new);
        tmp(1:c_rows, 1:c_cols) = nml_constant_cache;
        nml_constant_cache = tmp;
    end

    if m == 1
        
        C = 1;
        
    elseif m == 2
        
        % Cases k=0 and k=n both contribute 1 to the sum.
        C = 2;
        
        % The other cases (this could be further optimized by noting
        % that the terms for k=1 and k=n-1 are the same, etc.)
        for k = 1:(n-1)
            % Computations in log-space to avoid Matlab's warnings
            % about possible loss of precision.
            tmp = gammaln(n+1) - gammaln(k+1) - gammaln(n-k+1) ...
                  + k*log(k/n) + (n-k)*log(1-k/n);
            C = C + exp(tmp);
        end
        
    else
        
        a = nml_constant(n, 1);
        b = nml_constant(n, 2);
        
        for i = 3:m
            tmp = b + n / (m-2) * a;
            a = b;
            b = tmp;
        end
        
        C = b;
        
    end
    
    nml_constant_cache(n, m) = C;
    
end