function [] = q2b()

    num_vars = 6;
    permutation = randperm(num_vars);
    
    max_no_improvement = 10;
    num_random_restarts = 10;
    tabu_list = permutation;

    really_best_codelen = Inf;
    really_best_parents = {};
    
    best_codelen = Inf;
    best_parents = {};
    best_permutation = permutation;

    random_restart_count = 0;
    no_improvement_count = 0;
    
    while true
        
        fprintf('\n');
        
        old_best_codelen = best_codelen;
        
        % try out all permutations reachable by one adjacent-pair swap.
        % pick the best one (that is not in the tabu list)
        best_codelen = Inf;
        for i = 2:num_vars
            fprintf('.');
            
            new_permutation = permutation;
            new_permutation([i-1, i]) = new_permutation([i, i-1]);
            
            % check if new_permutation is in the tabu list
            tabu_list_cnt = size(tabu_list, 1);
            found = false;
            for j = 1:tabu_list_cnt
                if all(tabu_list(j,:) == new_permutation)
                    found = true;
                    break;
                end
            end
            if found
                continue;
            end

            [codelen, parents] = optimize_parents(num_vars, new_permutation);
            if codelen < best_codelen
                best_codelen = codelen;
                best_parents = parents;
                best_permutation = new_permutation;
            end
        end
        
        permutation = best_permutation;
        tabu_list = [ tabu_list; permutation ];
        
        fprintf('\nCode-length: %.2f bits\n', best_codelen);
        fprintf('Permutation: %s\n', mat2str(permutation));
        fprintf('Parent sets:\n');
        for i = 1:num_vars
            fprintf('Variable %d:\t%s\n', i-1, mat2str(best_parents{i} - 1));
        end
               
        if best_codelen < really_best_codelen
            really_best_codelen = best_codelen;
            really_best_parents = best_parents;
        end
        
        if old_best_codelen <= best_codelen
            fprintf('No improvement with this one.\n');
            no_improvement_count = no_improvement_count + 1;
        else
            no_improvement_count = 0;
        end
        
        if no_improvement_count == max_no_improvement
            fprintf('No improvements during the last %d steps.\n', ...
                max_no_improvement);
            
            no_improvement_count = 0;            
            random_restart_count = random_restart_count + 1;
            
            if random_restart_count <= num_random_restarts
                fprintf('Doing a random restart (%d/%d).\n', ...
                    random_restart_count, num_random_restarts);
                
                permutation = randperm(num_vars);
                tabu_list = [ tabu_list; permutation ];
                best_codelen = Inf;
            else
                fprintf('No more random restarts.\n');
                break;
            end
        end
        
    end

    fprintf('Finished.\n\n');
    fprintf('Final result:\n');
    fprintf('Code-length: %.2f bits.\n', really_best_codelen);
    fprintf('Parent sets:\n');
    for i = 1:num_vars
        fprintf('Variable %d:\t%s\n', i-1, ...
                mat2str(really_best_parents{i} - 1));
    end
    fprintf('\n');
        
end

function [codelen, parents] = optimize_parents(num_vars, permutation)
    
    % initialization
    parents = cell(num_vars, 1);
    for i = 1:num_vars
        parents{i} = [];
    end
    
    % First, parents{permutation(1)} is always [].
    % Second, parents{permutation(2)} can be either [] or [permutation(1)].
    codelen_without = q2_codelen(parents);
    parents{permutation(2)} = permutation(1);
    codelen_with = q2_codelen(parents);
    if codelen_without < codelen_with
        parents{permutation(2)} = [];
    end
    
    % for the rest of the variables, there are more options
    for i = 3:num_vars
        ii = permutation(i);
        best_codelen = Inf;
        best_parentset = [];
        for k = 0:(i-1)
            pss = nchoosek(permutation(1:(i-1)), k);
            pss_cnt = size(pss, 1);
            for j = 1:pss_cnt
                parents{ii} = pss(j, :);
                
                % fix strange matlab behavior
                if isempty(parents{ii})
                    parents{ii} = [];
                end
                
                codelen = q2_codelen(parents);
                if codelen < best_codelen
                    best_codelen = codelen;
                    best_parentset = parents{ii};
                end
            end
        end
        parents{ii} = best_parentset;
    end
    
    codelen = best_codelen;
        
end